require 'net/http'
require 'uri'
require 'json'

module MrJenkins

  class Client

    attr_accessor :authtoken
    attr_accessor :server
    attr_accessor :debug
    attr_accessor :jobName

    def initialize(opts)
      @debug = opts[:debug] || false
      puts __FILE__ + (__LINE__).to_s + " => #{opts.to_s}" if @debug
      @server = opts[:server]
      @authtoken = opts[:token].to_s
      @jobName = opts[:jobname].to_s

      MrJenkins::Jenkins.instance.config( { server: @server, auth: @authtoken , jobName: @jobName })
    end

    def summary(jobName)
      MrJenkins::Jenkins.instance.allStatus(jobName)
    end

    def _getUserID(buildInfo)
      user=''

      buildInfo['actions'].each do |actions|
        if actions.has_key?('causes') && actions['causes'][0]['userId']
          user = actions['causes'][0]['userId']
        end
      end

      user
    end


    # cli.toSlack(user, jobName, buildInfo['number'])
    def toSlack(opts)
      jobName = opts[:jobName].to_s
      buildID = opts[:buildID].to_s
      user = opts[:user].to_s

      buildInfo = nil

      if buildID.nil? || buildID.empty?
        buildInfo  = MrJenkins::Jenkins.instance.lastBuild(jobName)
        user = _getUserID(buildInfo)
        buildID = buildInfo['number'].to_s
      end

      slackRecord={}
      slackRecord['__custom__'] = {}

      slackRecord['__custom__'][:testType] = MrJenkins::Jenkins.instance.getTestType(buildInfo)
      slackRecord[:channel]='#scoutui-tmp'
      slackRecord[:user] = user

      response = MrJenkins::Jenkins.instance.jobStatus(jobName, buildID)
      summaryJson = summary(jobName)

      slackRecord[:jobStatus] = response
      slackRecord[:summary] = summaryJson


     puts "RESPONSE => " + JSON.pretty_generate(response) if @debug

      jenkinsResult = response['result'].to_s
      displayName = response['fullDisplayName'].to_s

      consoleText = MrJenkins::Jenkins.instance.console(jobName, buildID)

      if MrJenkins::Utils.isVerbose
        puts __FILE__ + (__LINE__).to_s + " console =>"
        puts consoleText.to_s
      end

      passed = consoleText.scan(/^.*\d+\s*passed\s*$/).flatten
      specs = consoleText.scan(/^\s*\d+\s*spec[s]*\s*.*/).flatten

      assertErrors = []


      if MrJenkins::Utils.isVerbose
        puts "User: #{user}"
        puts "#{displayName} execution is #{jenkinsResult}"
      end


      if specs.size > 0
        puts "Specs: #{specs.size.to_s}" if MrJenkins::Utils.isVerbose
        specs.each do |s|
          puts " => #{s.strip.to_s}" if MrJenkins::Utils.isVerbose
        end
        assertErrors = consoleText.scan(/^\s*AssertionError:.*/).flatten
      end


      fields=[]
      puts "Passed:" if MrJenkins::Utils.isVerbose
      passed.each do |s|
        _s = s.gsub(/I\/launcher\s*\-\s*/,'')
        puts " => #{_s.to_s}" if MrJenkins::Utils.isVerbose

        fields << { "title" => _s.to_s, "value" => "Passed", "short" => true }
      end

      slackRecord[:fields] = fields

      if assertErrors.size > 0
        puts "Assert Failures:"  if MrJenkins::Utils.isVerbose
        i=0
        assertErrors.each do |s|
          puts "#{i}. #{s.strip.to_s}" if MrJenkins::Utils.isVerbose
          fields << { "title" => s.strip.to_s, "value" => "Failed", "short" => true }
          i+=1
        end
      end

      slackCli = MrJenkins::Slack.new()
      slackCli.send(slackRecord,'pass')
    end


    def execute(jobname=nil)
      puts __FILE__ + (__LINE__).to_s +  "[execute]: #{jobname}" if MrJenkins::Utils.isVerbose

      jobName = !jobname.nil? ? jobname : @jobName
      buildInfo = MrJenkins::Jenkins.instance.lastBuild(jobName)

      if MrJenkins::Utils.isVerbose
        puts "== BUILD INFO =="
        puts JSON.pretty_generate(buildInfo)
      end

      user=''

      buildInfo['actions'].each do |actions|
        if actions.has_key?('causes') && actions['causes'][0]['userId']
          user = actions['causes'][0]['userId']
        end
      end

      puts "USER : #{user}" if MrJenkins::Utils.isVerbose

      jobStatus = MrJenkins::Jenkins.instance.jobStatus(jobName)

      if MrJenkins::Utils.isVerbose
        puts "JobStatus:"
        puts jobStatus
      end

# cli.toSlack( { user: user, jobName: jobName, buildID: buildInfo['number'] })
      toSlack( { jobName: jobName})
    end


  end

end


