require 'net/http'
require 'uri'
require 'json'
require 'singleton'

module MrJenkins

  class Jenkins
    include Singleton

    attr_accessor :authtoken
    attr_accessor :debug
    attr_accessor :server

    def config( opts = {server: s, auth: a, debug: false })
      puts __FILE__ + (__LINE__).to_s + " config() => #{opts.to_s}" if MrJenkins::Utils.isVerbose
      @server = opts[:server]
      @authtoken = opts[:auth]
      @debug = opts[:debug]
      @jobName = opts[:jobName]
    end

    def setAuthToken(a)
      @authtoken = a
    end

    def setServer(s)
      @server = s
    end

    def getJobName()
      @jobName
    end

    #
    # Get the Jenkins parameter 'TestType' - Functional, Visual
    def getTestType(buildInfo)
      testType = 'Functional'

      buildInfo['actions'].each do |actions|
        if actions.has_key?('parameters')
          actions['parameters'].each do |p|
            if p.has_key?('name') && p['name'].match(/testtype/)
              testType = p['value']
              break
            end
          end
        end
      end

      testType
    end


    def sendReq(uri, _parse=true)
      request = Net::HTTP::Get.new(uri)
      request["Authorization"] = "Basic #{@authtoken}"

      req_options = {
        use_ssl: uri.scheme == "https",
      }

      response = Net::HTTP.start(uri.hostname, uri.port, req_options) do |http|
        http.request(request)
      end

      _parse ? JSON.parse(response.body) : response.body
    end

    # TODO: Implement
    def buildJob(jobName)
      uri = URI.parse("#{@server}/job/" + URI::encode(jobName) + "/build")
      MrJenkins::Jenkins.instance.sendReq(uri)
    end


    def console(jobName, build)
      uri = URI.parse("#{@server}/job/" + URI::encode(jobName) + "/#{build}/consoleText/api/json")

      puts __FILE__ + (__LINE__).to_s + " Jenkins.console(#{jobName}, #{build}) => #{uri.to_s}" if MrJenkins::Utils.isVerbose
      MrJenkins::Jenkins.instance.sendReq(uri, false)
    end


    def getCrumb()
      uri = URI.parse("#{@server}/crumbIssuer/api/xml\?xpath\=concat\(//crumbRequestField,%22:%22,//crumb\)")
      MrJenkins::Jenkins.instance.sendReq(uri)
    end

    def allStatus(jobName)
      uri = URI.parse("#{@server}/job/" + URI::encode(jobName) + "/api/json")
      MrJenkins::Jenkins.instance.sendReq(uri)
    end


    def jobStatus(jobName, build=nil)
      id = build
      if build.nil?
        _b = MrJenkins::Jenkins.instance.lastBuild(jobName)
        id = _b['number']
      end
      uri = URI.parse("#{@server}/job/" + URI::encode(jobName) + "/#{id}/api/json")
      MrJenkins::Jenkins.instance.sendReq(uri)
    end


    def lastBuild(jobName=nil)

      jobName = jobName.nil? ? MrJenkins::Jenkins.instance.getJobName() : jobName

      uri = URI.parse("#{@server}/job/" + URI::encode(jobName) + "/lastBuild/api/json")

      puts __FILE__ + (__LINE__).to_s + " Jenkins.lastBuild(#{jobName}) => #{uri.to_s}" if MrJenkins::Utils.isVerbose
      MrJenkins::Jenkins.instance.sendReq(uri, true)
    end


  end

end
