
require 'net/http'
require 'uri'
require 'json'

module MrJenkins

  class Slack

    attr_accessor :token

    def msgTemplate()
      msg = <<-FOO
      [
                 {
                  "fallback": "Required plain-text summary of the attachment.",
                  "color": "#36a64f",
                  "pretext": "__PRETEXT__",
                  "author_name": "jenkins",
                  "author_link": "http://flickr.com/bobby/",
                  "author_icon": "http://flickr.com/icons/bobby.jpg",
                  "title": "__TITLE__",
                  "title_link": "__TITLE_LINK__",
                  "text": "Jenkins-Job",
                  "fields": [
                      {
                          "title": "Priority",
                          "value": "High",
                          "short": false
                      },
                      {
                          "title": "Second",
                          "value": "Medium",
                          "short": true
                      }
                  ],
                  "image_url": "http://10.253.15.88:8080/userContent/angry.jenkins.png",
                  "ximage_url": "http://my-website.com/path/to/image.jpg",
                  "xthumb_url": "http://example.com/path/to/thumb.png",
                  "xthumb_url": "https://a.slack-edge.com/205a/img/services/jenkins-ci_72.png",
                  "xthumb_url": "https://stark-bastion-95510.herokuapp.com/m3.jpg",
                  "thumb_url": "http://emojis.slackmojis.com/emojis/images/1508257596/3045/jenkins-party.gif?1508257596",
                  "footer": "*COREUI HIG*",
                  "footer_icon": "https://platform.slack-edge.com/img/default_application_icon.png",
                  "ts": 123456789,
                  "mrkdwn": true
              }
      ]
FOO
      JSON.parse(msg)
    end


    def initialize(opt={})
      @debug = opt[:debug] || false
      @token = ENV['SLACK_TOKEN']
    end

    def remove_html_tags(s)
      return '' if s.nil? || s.empty?
      re = /<("[^"]*"|'[^']*'|[^'">])*>/
      s.gsub!(re, '')
    end


    def _getUserID(buildInfo)
      user=''

      buildInfo['actions'].each do |actions|
        if actions.has_key?('causes') && actions['causes'][0]['userId']
          user = actions['causes'][0]['userId']
        end
      end

      user
    end

    def send(info, status)
      red = '#f40434'
      color = 'good'

      uri = URI.parse("https://slack.com/api/chat.postMessage")
      request = Net::HTTP::Post.new(uri)

      j = msgTemplate()

      # Custom
      if info.has_key?('__custom__')

        puts __FILE__ + (__LINE__).to_s + " CUSTOM => #{info['__custom__'].to_s}"  if @debug

        testType = info['__custom__'][:testType]
        if testType

          if testType.match(/accessibility/i)
            j[0]['footer_icon']='http://www.3cbilisim.com/images/wcag.png'
          end


        end
      end

      j[0]['pretext'] = info[:jobStatus]['fullDisplayName'].to_s + " - " + info[:jobStatus]['result'].to_s
      j[0]['title'] = '*' + info[:jobStatus]['fullDisplayName'].to_s + '*'

      ## Timestamp
      j[0]['ts'] = info[:jobStatus]['timestamp']

      j[0]['title_link'] = info[:jobStatus]['url']
      j[0]['author_name'] = info[:user]
      j[0]['mrkdwn'] = true
      j[0]['text'] = remove_html_tags( info[:summary]['description'])
      j[0]['fields'] = info[:fields]

      puts __FILE__ + (__LINE__).to_s + " => #{j.to_s}"  if @debug

      if info[:jobStatus].nil?
        puts __FILE__ + (__LINE__).to_s + " jobStatus => #{j.to_s}"
      end

      if info[:jobStatus]['result'].match(/fail/i)
        color = 'danger'
        j[0]['thumb_url'] = 'http://emojis.slackmojis.com/emojis/images/1476961898/1258/jenkins_devil.png?1476961898'
      end

      if info[:jobStatus]['result'].match(/fail/i)
        color = 'danger'
      end

      j[0]['color'] = color

      j2 = URI.escape(j.to_json)

      puts __FILE__ + (__LINE__).to_s + " j           => #{j2.to_s}" if @debug

      request.body = "token=#{@token}&channel=#{info[:channel]}&text=Text here.&username=#{info[:user]}&attachments=#{j2}"

      req_options = {
        use_ssl: uri.scheme == "https",
      }

      response = Net::HTTP.start(uri.hostname, uri.port, req_options) do |http|
        http.request(request)
      end
    end

  end

end
